#include <stdio.h>
#include <math.h>
#include <time.h>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include "buttons.h"

void waitForInput(void);
void quitApp(int exitCode);
void initialDraw(void);
void displayTime(void);
void startStopwatch(void);
void displayGuideText(char text[]);
void eventHandler(void);
int initLibs(void);

int watchRunning = 0;

long elapsedTime = 0;
char elapsedTimeString[256];

int width, height, left, top;
int hours, minutes, seconds, fractions;

SDL_Surface *screen = NULL;
SDL_Surface *text_surface = NULL;
SDL_Surface *help_surface = NULL;

SDL_Color color = {255,255,255};
TTF_Font *font = NULL;
TTF_Font *helpFont = NULL;

int main(int argc, char *argv[]) {
	if (initLibs() != 0) {
		quitApp(1); // Something failed to initialize, quit the application
	}
	
	initialDraw();
	
	while(1) {
		displayGuideText("[A] - Start/Stop, [R] - Reset, [SELECT] - Quit");
		waitForInput();
	}
	
	return 0;
}

int initLibs(void) {
	SDL_Init(SDL_INIT_VIDEO); // Initialize SDL
	screen = SDL_SetVideoMode(320, 240, 16, SDL_SWSURFACE); // Set screen video mode 
	int ttfresult = TTF_Init(); // Initialize SDL_ttf
	SDL_ShowCursor(SDL_DISABLE); // No need for cursor on a mouse-less device
	
	if(screen == NULL) { // Did SDL load?
		printf("Couldn't set video mode:\n%s\n\nPress [START] to exit.\n", SDL_GetError());
		getchar();
		return 1;
	}
	
	if(ttfresult == -1) { // Did SDL_ttf load?
		printf("TTF_Init:\n%s\n\nPress [START] to exit.\n", TTF_GetError());
		getchar();
		return 1;
	}
		
	font = TTF_OpenFont("font.ttf", 48);
	helpFont = TTF_OpenFont("font.ttf", 12);
	
	if(!font) { // Did the font open?
		printf("TTF_OpenFont:\n%s\n\nPress [START] to exit.\n", TTF_GetError());
		getchar();
		return 1;
	}
	
	if(!helpFont) { // Did the font open?
		printf("TTF_OpenFont:\n%s\n\nPress [START] to exit.\n", TTF_GetError());
		getchar();
		return 1;
	}
	
	return 0;
}

void initialDraw(void) {
	sprintf(elapsedTimeString, "%s", "00:00:00");
	TTF_SizeText(font, elapsedTimeString, &width, &height);
	
	left = ((320 - width) / 2);
	top = ((240 - height) / 2);
	
	SDL_Rect destination;
	destination.x = left;
	destination.y = top;
	destination.w = width;
	destination.h = height;
	
	text_surface = TTF_RenderText_Blended(font, elapsedTimeString, color);
	SDL_FillRect(screen, NULL, 0x0000);
	
	displayGuideText("[A] - Start/Stop, [R] - Reset, [SELECT] - Quit");
	SDL_BlitSurface(text_surface, NULL, screen, &destination);
	
	SDL_Flip(screen);
}

void displayTime(void) {
	hours = floor(elapsedTime / 3600);
	minutes = floor((elapsedTime - (hours * 3600)) / 60);
	seconds = floor(elapsedTime % 60);
	
	sprintf(elapsedTimeString, "%02d:%02d:%02d", hours, minutes, seconds);
	TTF_SizeText(font, elapsedTimeString, &width, &height);
	
	left = ((320 - width) / 2);
	top = ((240 - height) / 2);
	
	SDL_Rect destination;
	destination.x = left;
	destination.y = top;
	destination.w = width;
	destination.h = height;
	
	text_surface = TTF_RenderText_Blended(font, elapsedTimeString, color);
	
	SDL_FillRect(screen, NULL, 0x0000);
	SDL_BlitSurface(text_surface, NULL, screen, &destination);
	displayGuideText("[A] - Start/Stop, [R] - Reset, [SELECT] - Quit");
	SDL_Flip(screen);
}

void startStopwatch(void) {
	while(watchRunning == 1) {
		elapsedTime += 1;
		displayTime();
		SDL_Delay(1000);
		eventHandler();
	}
}

void eventHandler(void) {
	SDL_Event event;
	while (SDL_PollEvent(&event)) {      
		if (event.type == SDL_QUIT) {
			quitApp(0);
		} else if(event.type == SDL_KEYDOWN) {
			switch(event.key.keysym.sym) {
				case DINGOO_BUTTON_SELECT: // Quit
					quitApp(0);
					break;
				case DINGOO_BUTTON_R: // Reset
					elapsedTime = 0;
					displayTime();
					break;
				case DINGOO_BUTTON_A: // Stop
					if(watchRunning == 0) {
						watchRunning = 1;
						startStopwatch();
					} else {
						watchRunning = 0;
					}
					break;
				default:
					break;
			}
		}
	}
}

void displayGuideText(char *text) {
	int w, h, left;
	TTF_SizeText(helpFont, text, &w, &h);
	
	help_surface = TTF_RenderText_Blended(helpFont, text, color);
	
	left = ((320 - w) / 2);
	
	SDL_Rect dest;
	dest.x = left;
	dest.y = 0;
	dest.w = w;
	dest.h = h;
	
	SDL_BlitSurface(help_surface, NULL, screen, &dest);
}

void waitForInput(void) {
	// Wait for user to press a key...
	SDL_Delay(10);
	eventHandler();
}

void quitApp(int exitCode) {
	// Close SDL_ttf
	SDL_FreeSurface(text_surface);
	TTF_CloseFont(font);
	font = NULL;
	TTF_Quit();
	
	// Close SDL
	SDL_FreeSurface(screen);
	screen = NULL;
	SDL_Quit();
	
	exit(exitCode);
}
